// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/ids/v1/ids.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_IDS_V1_IDS_CONNECTION_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_IDS_V1_IDS_CONNECTION_H

#include "google/cloud/ids/v1/ids_connection_idempotency_policy.h"
#include "google/cloud/ids/v1/internal/ids_retry_traits.h"
#include "google/cloud/backoff_policy.h"
#include "google/cloud/future.h"
#include "google/cloud/internal/retry_policy_impl.h"
#include "google/cloud/no_await_tag.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/stream_range.h"
#include "google/cloud/version.h"
#include <google/cloud/ids/v1/ids.pb.h>
#include <google/longrunning/operations.grpc.pb.h>
#include <memory>

namespace google {
namespace cloud {
namespace ids_v1 {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

/// The retry policy for `IDSConnection`.
class IDSRetryPolicy : public ::google::cloud::RetryPolicy {
 public:
  /// Creates a new instance of the policy, reset to the initial state.
  virtual std::unique_ptr<IDSRetryPolicy> clone() const = 0;
};

/**
 * A retry policy for `IDSConnection` based on counting errors.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - More than a prescribed number of transient failures is detected.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class IDSLimitedErrorCountRetryPolicy : public IDSRetryPolicy {
 public:
  /**
   * Create an instance that tolerates up to @p maximum_failures transient
   * errors.
   *
   * @note Disable the retry loop by providing an instance of this policy with
   *     @p maximum_failures == 0.
   */
  explicit IDSLimitedErrorCountRetryPolicy(int maximum_failures)
      : impl_(maximum_failures) {}

  IDSLimitedErrorCountRetryPolicy(
      IDSLimitedErrorCountRetryPolicy&& rhs) noexcept
      : IDSLimitedErrorCountRetryPolicy(rhs.maximum_failures()) {}
  IDSLimitedErrorCountRetryPolicy(
      IDSLimitedErrorCountRetryPolicy const& rhs) noexcept
      : IDSLimitedErrorCountRetryPolicy(rhs.maximum_failures()) {}

  int maximum_failures() const { return impl_.maximum_failures(); }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<IDSRetryPolicy> clone() const override {
    return std::make_unique<IDSLimitedErrorCountRetryPolicy>(
        maximum_failures());
  }

  // This is provided only for backwards compatibility.
  using BaseType = IDSRetryPolicy;

 private:
  google::cloud::internal::LimitedErrorCountRetryPolicy<
      ids_v1_internal::IDSRetryTraits>
      impl_;
};

/**
 * A retry policy for `IDSConnection` based on elapsed time.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - The elapsed time in the retry loop exceeds a prescribed duration.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class IDSLimitedTimeRetryPolicy : public IDSRetryPolicy {
 public:
  /**
   * Constructor given a `std::chrono::duration<>` object.
   *
   * @tparam DurationRep a placeholder to match the `Rep` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the underlying
   *     arithmetic type used to store the number of ticks. For our purposes it
   *     is simply a formal parameter.
   * @tparam DurationPeriod a placeholder to match the `Period` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the length of the
   *     tick in seconds, expressed as a `std::ratio<>`. For our purposes it is
   *     simply a formal parameter.
   * @param maximum_duration the maximum time allowed before the policy expires.
   *     While the application can express this time in any units they desire,
   *     the class truncates to milliseconds.
   *
   * @see https://en.cppreference.com/w/cpp/chrono/duration for more information
   *     about `std::chrono::duration`.
   */
  template <typename DurationRep, typename DurationPeriod>
  explicit IDSLimitedTimeRetryPolicy(
      std::chrono::duration<DurationRep, DurationPeriod> maximum_duration)
      : impl_(maximum_duration) {}

  IDSLimitedTimeRetryPolicy(IDSLimitedTimeRetryPolicy&& rhs) noexcept
      : IDSLimitedTimeRetryPolicy(rhs.maximum_duration()) {}
  IDSLimitedTimeRetryPolicy(IDSLimitedTimeRetryPolicy const& rhs) noexcept
      : IDSLimitedTimeRetryPolicy(rhs.maximum_duration()) {}

  std::chrono::milliseconds maximum_duration() const {
    return impl_.maximum_duration();
  }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<IDSRetryPolicy> clone() const override {
    return std::make_unique<IDSLimitedTimeRetryPolicy>(maximum_duration());
  }

  // This is provided only for backwards compatibility.
  using BaseType = IDSRetryPolicy;

 private:
  google::cloud::internal::LimitedTimeRetryPolicy<
      ids_v1_internal::IDSRetryTraits>
      impl_;
};

/**
 * The `IDSConnection` object for `IDSClient`.
 *
 * This interface defines virtual methods for each of the user-facing overload
 * sets in `IDSClient`. This allows users to inject custom behavior
 * (e.g., with a Google Mock object) when writing tests that use objects of type
 * `IDSClient`.
 *
 * To create a concrete instance, see `MakeIDSConnection()`.
 *
 * For mocking, see `ids_v1_mocks::MockIDSConnection`.
 */
class IDSConnection {
 public:
  virtual ~IDSConnection() = 0;

  virtual Options options() { return Options{}; }

  virtual StreamRange<google::cloud::ids::v1::Endpoint> ListEndpoints(
      google::cloud::ids::v1::ListEndpointsRequest request);

  virtual StatusOr<google::cloud::ids::v1::Endpoint> GetEndpoint(
      google::cloud::ids::v1::GetEndpointRequest const& request);

  virtual future<StatusOr<google::cloud::ids::v1::Endpoint>> CreateEndpoint(
      google::cloud::ids::v1::CreateEndpointRequest const& request);

  virtual StatusOr<google::longrunning::Operation> CreateEndpoint(
      NoAwaitTag, google::cloud::ids::v1::CreateEndpointRequest const& request);

  virtual future<StatusOr<google::cloud::ids::v1::Endpoint>> CreateEndpoint(
      google::longrunning::Operation const& operation);

  virtual future<StatusOr<google::cloud::ids::v1::OperationMetadata>>
  DeleteEndpoint(google::cloud::ids::v1::DeleteEndpointRequest const& request);

  virtual StatusOr<google::longrunning::Operation> DeleteEndpoint(
      NoAwaitTag, google::cloud::ids::v1::DeleteEndpointRequest const& request);

  virtual future<StatusOr<google::cloud::ids::v1::OperationMetadata>>
  DeleteEndpoint(google::longrunning::Operation const& operation);
};

/**
 * A factory function to construct an object of type `IDSConnection`.
 *
 * The returned connection object should not be used directly; instead it
 * should be passed as an argument to the constructor of IDSClient.
 *
 * The optional @p options argument may be used to configure aspects of the
 * returned `IDSConnection`. Expected options are any of the types in
 * the following option lists:
 *
 * - `google::cloud::CommonOptionList`
 * - `google::cloud::GrpcOptionList`
 * - `google::cloud::UnifiedCredentialsOptionList`
 * - `google::cloud::ids_v1::IDSPolicyOptionList`
 *
 * @note Unexpected options will be ignored. To log unexpected options instead,
 *     set `GOOGLE_CLOUD_CPP_ENABLE_CLOG=yes` in the environment.
 *
 * @param options (optional) Configure the `IDSConnection` created by
 * this function.
 */
std::shared_ptr<IDSConnection> MakeIDSConnection(Options options = {});

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace ids_v1
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_IDS_V1_IDS_CONNECTION_H
