// Copyright 2023 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/redis/cluster/v1/cloud_redis_cluster.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_REDIS_CLUSTER_V1_CLOUD_REDIS_CLUSTER_CONNECTION_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_REDIS_CLUSTER_V1_CLOUD_REDIS_CLUSTER_CONNECTION_H

#include "google/cloud/redis/cluster/v1/cloud_redis_cluster_connection_idempotency_policy.h"
#include "google/cloud/redis/cluster/v1/internal/cloud_redis_cluster_retry_traits.h"
#include "google/cloud/backoff_policy.h"
#include "google/cloud/future.h"
#include "google/cloud/internal/retry_policy_impl.h"
#include "google/cloud/no_await_tag.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/stream_range.h"
#include "google/cloud/version.h"
#include <google/cloud/redis/cluster/v1/cloud_redis_cluster.pb.h>
#include <google/longrunning/operations.grpc.pb.h>
#include <memory>

namespace google {
namespace cloud {
namespace redis_cluster_v1 {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

/// The retry policy for `CloudRedisClusterConnection`.
class CloudRedisClusterRetryPolicy : public ::google::cloud::RetryPolicy {
 public:
  /// Creates a new instance of the policy, reset to the initial state.
  virtual std::unique_ptr<CloudRedisClusterRetryPolicy> clone() const = 0;
};

/**
 * A retry policy for `CloudRedisClusterConnection` based on counting errors.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - More than a prescribed number of transient failures is detected.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class CloudRedisClusterLimitedErrorCountRetryPolicy
    : public CloudRedisClusterRetryPolicy {
 public:
  /**
   * Create an instance that tolerates up to @p maximum_failures transient
   * errors.
   *
   * @note Disable the retry loop by providing an instance of this policy with
   *     @p maximum_failures == 0.
   */
  explicit CloudRedisClusterLimitedErrorCountRetryPolicy(int maximum_failures)
      : impl_(maximum_failures) {}

  CloudRedisClusterLimitedErrorCountRetryPolicy(
      CloudRedisClusterLimitedErrorCountRetryPolicy&& rhs) noexcept
      : CloudRedisClusterLimitedErrorCountRetryPolicy(rhs.maximum_failures()) {}
  CloudRedisClusterLimitedErrorCountRetryPolicy(
      CloudRedisClusterLimitedErrorCountRetryPolicy const& rhs) noexcept
      : CloudRedisClusterLimitedErrorCountRetryPolicy(rhs.maximum_failures()) {}

  int maximum_failures() const { return impl_.maximum_failures(); }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<CloudRedisClusterRetryPolicy> clone() const override {
    return std::make_unique<CloudRedisClusterLimitedErrorCountRetryPolicy>(
        maximum_failures());
  }

  // This is provided only for backwards compatibility.
  using BaseType = CloudRedisClusterRetryPolicy;

 private:
  google::cloud::internal::LimitedErrorCountRetryPolicy<
      redis_cluster_v1_internal::CloudRedisClusterRetryTraits>
      impl_;
};

/**
 * A retry policy for `CloudRedisClusterConnection` based on elapsed time.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - The elapsed time in the retry loop exceeds a prescribed duration.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class CloudRedisClusterLimitedTimeRetryPolicy
    : public CloudRedisClusterRetryPolicy {
 public:
  /**
   * Constructor given a `std::chrono::duration<>` object.
   *
   * @tparam DurationRep a placeholder to match the `Rep` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the underlying
   *     arithmetic type used to store the number of ticks. For our purposes it
   *     is simply a formal parameter.
   * @tparam DurationPeriod a placeholder to match the `Period` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the length of the
   *     tick in seconds, expressed as a `std::ratio<>`. For our purposes it is
   *     simply a formal parameter.
   * @param maximum_duration the maximum time allowed before the policy expires.
   *     While the application can express this time in any units they desire,
   *     the class truncates to milliseconds.
   *
   * @see https://en.cppreference.com/w/cpp/chrono/duration for more information
   *     about `std::chrono::duration`.
   */
  template <typename DurationRep, typename DurationPeriod>
  explicit CloudRedisClusterLimitedTimeRetryPolicy(
      std::chrono::duration<DurationRep, DurationPeriod> maximum_duration)
      : impl_(maximum_duration) {}

  CloudRedisClusterLimitedTimeRetryPolicy(
      CloudRedisClusterLimitedTimeRetryPolicy&& rhs) noexcept
      : CloudRedisClusterLimitedTimeRetryPolicy(rhs.maximum_duration()) {}
  CloudRedisClusterLimitedTimeRetryPolicy(
      CloudRedisClusterLimitedTimeRetryPolicy const& rhs) noexcept
      : CloudRedisClusterLimitedTimeRetryPolicy(rhs.maximum_duration()) {}

  std::chrono::milliseconds maximum_duration() const {
    return impl_.maximum_duration();
  }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<CloudRedisClusterRetryPolicy> clone() const override {
    return std::make_unique<CloudRedisClusterLimitedTimeRetryPolicy>(
        maximum_duration());
  }

  // This is provided only for backwards compatibility.
  using BaseType = CloudRedisClusterRetryPolicy;

 private:
  google::cloud::internal::LimitedTimeRetryPolicy<
      redis_cluster_v1_internal::CloudRedisClusterRetryTraits>
      impl_;
};

/**
 * The `CloudRedisClusterConnection` object for `CloudRedisClusterClient`.
 *
 * This interface defines virtual methods for each of the user-facing overload
 * sets in `CloudRedisClusterClient`. This allows users to inject custom
 * behavior (e.g., with a Google Mock object) when writing tests that use
 * objects of type `CloudRedisClusterClient`.
 *
 * To create a concrete instance, see `MakeCloudRedisClusterConnection()`.
 *
 * For mocking, see `redis_cluster_v1_mocks::MockCloudRedisClusterConnection`.
 */
class CloudRedisClusterConnection {
 public:
  virtual ~CloudRedisClusterConnection() = 0;

  virtual Options options() { return Options{}; }

  virtual StreamRange<google::cloud::redis::cluster::v1::Cluster> ListClusters(
      google::cloud::redis::cluster::v1::ListClustersRequest request);

  virtual StatusOr<google::cloud::redis::cluster::v1::Cluster> GetCluster(
      google::cloud::redis::cluster::v1::GetClusterRequest const& request);

  virtual future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
  UpdateCluster(
      google::cloud::redis::cluster::v1::UpdateClusterRequest const& request);

  virtual StatusOr<google::longrunning::Operation> UpdateCluster(
      NoAwaitTag,
      google::cloud::redis::cluster::v1::UpdateClusterRequest const& request);

  virtual future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
  UpdateCluster(google::longrunning::Operation const& operation);

  virtual future<StatusOr<google::protobuf::Any>> DeleteCluster(
      google::cloud::redis::cluster::v1::DeleteClusterRequest const& request);

  virtual StatusOr<google::longrunning::Operation> DeleteCluster(
      NoAwaitTag,
      google::cloud::redis::cluster::v1::DeleteClusterRequest const& request);

  virtual future<StatusOr<google::protobuf::Any>> DeleteCluster(
      google::longrunning::Operation const& operation);

  virtual future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
  CreateCluster(
      google::cloud::redis::cluster::v1::CreateClusterRequest const& request);

  virtual StatusOr<google::longrunning::Operation> CreateCluster(
      NoAwaitTag,
      google::cloud::redis::cluster::v1::CreateClusterRequest const& request);

  virtual future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
  CreateCluster(google::longrunning::Operation const& operation);

  virtual StatusOr<google::cloud::redis::cluster::v1::CertificateAuthority>
  GetClusterCertificateAuthority(
      google::cloud::redis::cluster::v1::
          GetClusterCertificateAuthorityRequest const& request);
};

/**
 * A factory function to construct an object of type
 * `CloudRedisClusterConnection`.
 *
 * The returned connection object should not be used directly; instead it
 * should be passed as an argument to the constructor of
 * CloudRedisClusterClient.
 *
 * The optional @p options argument may be used to configure aspects of the
 * returned `CloudRedisClusterConnection`. Expected options are any of the types
 * in the following option lists:
 *
 * - `google::cloud::CommonOptionList`
 * - `google::cloud::GrpcOptionList`
 * - `google::cloud::UnifiedCredentialsOptionList`
 * - `google::cloud::redis_cluster_v1::CloudRedisClusterPolicyOptionList`
 *
 * @note Unexpected options will be ignored. To log unexpected options instead,
 *     set `GOOGLE_CLOUD_CPP_ENABLE_CLOG=yes` in the environment.
 *
 * @param options (optional) Configure the `CloudRedisClusterConnection` created
 * by this function.
 */
std::shared_ptr<CloudRedisClusterConnection> MakeCloudRedisClusterConnection(
    Options options = {});

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace redis_cluster_v1
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_REDIS_CLUSTER_V1_CLOUD_REDIS_CLUSTER_CONNECTION_H
